@@ -25,7 +25,7 @@ from rednotebook.util import utils
 from rednotebook import info
 from rednotebook.util import filesystem
 from rednotebook.util import markup
-from rednotebook.gui.imports import ImportAssistant
+#from rednotebook.gui.imports import ImportAssistant
 
 
 class MainMenuBar(object):
@@ -79,7 +79,6 @@ class MainMenuBar(object):
                 <menuitem action="Translate"/>
                 <menuitem action="ReportBug"/>
                 <separator/>
-                <menuitem action="Examples"/>
                 <menuitem action="Info"/>
             </menu>
         </menubar>
@@ -136,15 +135,10 @@ class MainMenuBar(object):
                 '<Ctrl><Alt>p', None, self.on_options_menuitem_activate),
 
             ('HelpMenu', None, _('_Help')),
-            #('CheckVersion', None, _('Check For New Version'),
-            #   None, _('Check for a new version now'), self.on_check_version_menu_item_activate),
             ('Help', gtk.STOCK_HELP, _('Contents'),
                 '<Ctrl>h', _('Open the RedNotebook documentation'), self.on_help_menu_item_activate),
             ('OnlineHelp', None, _('Get Help Online'),
-                None, _('Visit the RedNotebook forum'), self.on_online_help),
-            ('Examples', None, _('Restore example content'),
-                None, _('Fill some free days with example content. Do not overwrite anything'),
-                self.on_example_menu_item_activate),
+                None, _('Browse answered questions or ask a new one'), self.on_online_help),
             ('Translate', None, _('Translate RedNotebook'),
                 None, _('Connect to the Launchpad website to help translate RedNotebook'),
                 self.on_translate),
@@ -160,10 +154,6 @@ class MainMenuBar(object):
 
         # Add a UI description
         self.uimanager.add_ui_from_string(menu_xml)
-        
-        # We want to hide the examples button for now
-        examples_button = self.uimanager.get_widget('/MainMenuBar/HelpMenu/Examples')
-        examples_button.hide()
 
         # Create a Menu
         self.menubar = self.uimanager.get_widget('/MainMenuBar')
@@ -176,7 +166,6 @@ class MainMenuBar(object):
             for action in actions:
                 widgets = action.get_proxies()
                 tooltip = action.get_property('tooltip')
-                print widgets, tooltip
                 if tooltip:
                     for widget in widgets:
                         widget.set_tooltip_markup(tooltip)
@@ -225,7 +214,7 @@ class MainMenuBar(object):
 #       event.state = gtk.gdk.CONTROL_MASK
 #       self.main_frame.emit("key_press_event",event)
         self.main_window.day_text_field.day_text_view.emit('cut_clipboard')
-        
+
     def on_fullscreen_menuitem_activate(self, widget):
         self.main_window.toggle_fullscreen()
 
@@ -239,25 +228,21 @@ class MainMenuBar(object):
     def on_options_menuitem_activate(self, widget):
         self.main_window.options_manager.on_options_dialog()
 
-
     def on_backup_activate(self, widget):
-        self.journal.backup_contents(backup_file=self.main_window.get_backup_file())
-        
+        self.journal.archiver.backup()
+
     def on_import_menu_item_activate(self, widget):
         assistant = ImportAssistant(self.journal)
         assistant.run()
 
     def on_export_menu_item_activate(self, widget):
         self.journal.save_old_day()
-        
+
         self.main_window.export_assistant.run()
 
     def on_statistics_menu_item_activate(self, widget):
         self.journal.stats.show_dialog(self.main_window.stats_dialog)
 
-    def on_example_menu_item_activate(self, widget):
-        self.journal.add_instruction_content()
-
     def on_help_menu_item_activate(self, widget):
         temp_dir = self.journal.dirs.temp_dir
         filesystem.write_file(os.path.join(temp_dir, 'source.txt'), info.help_text)
@@ -266,23 +251,21 @@ class MainMenuBar(object):
         html = markup.convert(info.help_text, 'xhtml', headers, options)
         utils.show_html_in_browser(html, os.path.join(temp_dir, 'help.html'))
 
-    #def on_check_version_menu_item_activate(self, widget):
-    #   utils.check_new_version(self.main_window, info.version)
-    
     def on_online_help(self, widget):
-        webbrowser.open(info.forum_url)
-        
+        webbrowser.open(info.answers_url)
+
     def on_translate(self, widget):
         webbrowser.open(info.translation_url)
-        
+
     def on_report_bug(self, widget):
         webbrowser.open(info.bug_url)
 
     def on_info_activate(self, widget):
         self.info_dialog = self.main_window.builder.get_object('about_dialog')
+        self.info_dialog.set_transient_for(self.main_window.main_frame)
         self.info_dialog.set_name('RedNotebook')
         self.info_dialog.set_version(info.version)
-        self.info_dialog.set_copyright('Copyright (c) 2008 Jendrik Seipp')
+        self.info_dialog.set_copyright('Copyright (c) 2008-2011 Jendrik Seipp')
         self.info_dialog.set_comments(_('A Desktop Journal'))
         gtk.about_dialog_set_url_hook(lambda dialog, url: webbrowser.open(url))
         self.info_dialog.set_website(info.url)
