@@ -35,10 +35,9 @@ except ImportError:
 # The presence of the yaml module has been checked
 try:
     from yaml import CLoader as Loader
-    from yaml import CDumper as Dumper
-    #logging.info('Using libyaml for loading and dumping')
+    #from yaml import CDumper as Dumper
 except ImportError:
-    from yaml import Loader, Dumper
+    from yaml import Loader #, Dumper
     logging.info('Using pyyaml for loading and dumping')
 
 from rednotebook.data import Month
@@ -114,20 +113,24 @@ def save_months_to_disk(months, dir, frame, exit_imminent=False, changing_journa
         if month.edited or saveas:
             something_saved = True
             month_file_string = os.path.join(dir, year_and_month + '.txt')
-            with codecs.open(month_file_string, 'wb', encoding='utf-8') as month_file:
-                month_content = {}
-                for day_number, day in month.days.iteritems():
-                    # do not add empty days
-                    if not day.empty:
-                        month_content[day_number] = day.content
+            month_content = {}
+            for day_number, day in month.days.iteritems():
+                # do not add empty days
+                if not day.empty:
+                    month_content[day_number] = day.content
 
+            # Do not save empty month files
+            if not month_content and not os.path.exists(month_file_string):
+                continue
+
+            with codecs.open(month_file_string, 'wb', encoding='utf-8') as month_file:
                 try:
                     # yaml.dump(month_content, month_file, Dumper=Dumper)
                     # This version produces readable unicode and no python directives
                     yaml.safe_dump(month_content, month_file, allow_unicode=True)
                     month.edited = False
-                except OSError, err:
+                except OSError:
                     frame.show_save_error_dialog(exit_imminent)
-                except IOError, err:
+                except IOError:
                     frame.show_save_error_dialog(exit_imminent)
     return something_saved
