@@ -20,25 +20,35 @@
 from __future__ import with_statement
 
 import os
-import sys
 import logging
+import re
+import sys
 
 import pango
 import gobject
 
 # Testing
 if __name__ == '__main__':
-    import sys
     sys.path.insert(0, '../../')
     logging.basicConfig(level=logging.DEBUG,
                     format='%(levelname)-8s %(message)s',)
 
 from rednotebook.external import txt2tags
 from rednotebook.util import filesystem
-from rednotebook.util import dates
-from rednotebook.util import utils
 
 
+# Linebreaks are only allowed at line ends
+REGEX_LINEBREAK = r'\\\\[\s]*$'
+REGEX_HTML_LINK = r'<a.*?>(.*?)</a>'
+
+CSS = """\
+<style type="text/css">
+    body {
+        font-family: Ubuntu, sans-serif;
+    }
+</style>
+"""
+
 
 def convert_categories_to_markup(categories, with_category_title=True):
     'Only add Category title if the text is displayed'
@@ -61,9 +71,9 @@ def get_markup_for_day(day, with_text=True, categories=None, date=None):
     '''
     export_string = ''
 
-    # Add date
+    # Add date if it is not None and not the empty string
     if date:
-        export_string += '= ' + date + ' =\n\n'
+        export_string += '= %s =\n\n' % date
 
     # Add text
     if with_text:
@@ -73,7 +83,7 @@ def get_markup_for_day(day, with_text=True, categories=None, date=None):
     category_content_pairs = day.get_category_content_pairs()
 
     if categories:
-        categories = map(lambda word: word.lower(), categories)
+        categories = [word.lower() for word in categories]
         export_categories = dict((x,y) for (x, y) in category_content_pairs.items()
                         if x.lower() in categories)
     elif categories is None:
@@ -111,19 +121,19 @@ def _get_config(type):
     config['postproc'] = []
     config['preproc'] = []
 
+    # Allow line breaks, r'\\\\' are 2 \ for regexes
+    config['preproc'].append([REGEX_LINEBREAK, 'REDNOTEBOOKLINEBREAK'])
 
     if type == 'xhtml' or type == 'html':
-        config['encoding'] = 'UTF-8'       # document encoding
+        config['encoding'] = 'UTF-8'  # document encoding
         config['toc'] = 0
-        config['style'] = [os.path.join(filesystem.files_dir, 'stylesheet.css')]
-        config['css-inside'] = 1
         config['css-sugar'] = 1
 
-        # keepnote only recognizes "<strike>"
-        config['postproc'].append(['(?i)(</?)s>', r'\1strike>'])
+        # Custom css
+        config['postproc'].append([r'</head>', CSS + '</head>'])
 
-        # Allow line breaks, r'\\\\' are 2 \ for regexes
-        config['postproc'].append([r'\\\\', '<br />'])
+        # Line breaks
+        config['postproc'].append([r'REDNOTEBOOKLINEBREAK', '<br />'])
 
         # Apply image resizing
         config['postproc'].append([r'src=\"WIDTH(\d+)-', r'width="\1" src="'])
@@ -151,15 +161,15 @@ def _get_config(type):
         config['postproc'].append([r'htmladdnormallink\{(.*)\}\{%s(.*)\}' % scheme,
                                    r'htmladdnormallink{\1}{run:\2}'])
 
-        # Allow line breaks, r'\\\\' are 2 \ for regexes
-        config['postproc'].append([r'\$\\backslash\$\$\\backslash\$', r'\\\\'])
+        # Line breaks
+        config['postproc'].append([r'REDNOTEBOOKLINEBREAK', r'\\\\'])
 
         # Apply image resizing
         config['postproc'].append([r'includegraphics\{("?)WIDTH(\d+)-', r'includegraphics[width=\2px]{\1'])
 
     elif type == 'txt':
-        # Allow line breaks, r'\\\\' are 2 \ for regexes
-        config['postproc'].append([r'\\\\', '\n'])
+        # Line breaks
+        config['postproc'].append([r'REDNOTEBOOKLINEBREAK', '\n'])
 
         # Apply image resizing ([WIDTH400-file:///pathtoimage.jpg])
         config['postproc'].append([r'\[WIDTH(\d+)-(.+)\]', r'[\2?\1]'])
@@ -170,12 +180,13 @@ def _get_config(type):
     img_name = r'\S.*\S|\S'
 
     # Apply this prepoc only after the latex image quotes have been added
+    # TODO: Allow whitespace around [ and ]
     config['preproc'].append([r'\[(%s\.(%s))\?(\d+)\]' % (img_name, img_ext), r'[WIDTH\3-\1]'])
 
     return config
 
 
-def convert(txt, target, headers=None, options=None, append_whitespace=False):
+def convert(txt, target, headers=None, options=None):
     '''
     Code partly taken from txt2tags tarball
     '''
@@ -183,21 +194,6 @@ def convert(txt, target, headers=None, options=None, append_whitespace=False):
     # Here is the marked body text, it must be a list.
     txt = txt.split('\n')
 
-    '''
-    Without this HACK "First Line\n_second Line" is rendered to
-    "First LineSecond Line", but we want "First Line Second Line"
-
-    We only need this for the keepnote input, the exports work fine
-    '''
-    #TODO: Remove once keepnote files have been removed
-    if append_whitespace:
-        def add_whitespace(line):
-            if line.rstrip().endswith(r'\\'):
-                return line.rstrip()
-            else:
-                return line + ' '
-        txt = map(add_whitespace, txt)
-
     # Set the three header fields
     if headers is None:
         headers = ['', '', '']
@@ -230,9 +226,9 @@ def convert(txt, target, headers=None, options=None, append_whitespace=False):
     except:
         result = txt2tags.getUnknownErrorMessage()
         logging.error(result)
-
     return result
 
+
 def convert_to_pango(txt, headers=None, options=None):
     '''
     Code partly taken from txt2tags tarball
@@ -252,11 +248,13 @@ def convert_to_pango(txt, headers=None, options=None):
     config['target'] = 'xhtml'
 
     config['preproc'] = []
+    # We need to escape the ampersand here, otherwise "&amp;" would become
+    # "&amp;amp;"
     config['preproc'].append([r'&amp;', '&'])
 
-    # Allow line breaks, r'\\\\' are 2 \ for regexes
+    # Allow line breaks
     config['postproc'] = []
-    config['postproc'].append([r'\\\\', '\n'])
+    config['postproc'].append([REGEX_LINEBREAK, '\n'])
 
     if options is not None:
         config.update(options)
@@ -284,6 +282,12 @@ def convert_to_pango(txt, headers=None, options=None):
     logging.log(5, 'Converted "%s" text to "%s" txt2tags markup' %
                 (repr(original_txt), repr(result)))
 
+    # Remove unknown tags (<a>)
+    def replace_links(match):
+        """Return the link name."""
+        return match.group(1)
+    result = re.sub(REGEX_HTML_LINK, replace_links, result)
+
     try:
         attr_list, plain, accel = pango.parse_markup(result)
 
@@ -292,12 +296,7 @@ def convert_to_pango(txt, headers=None, options=None):
     except gobject.GError:
         # There are unknown tags in the markup, return the original text
         logging.debug('There are unknown tags in the markup: %s' % result)
-        # We have to take care of the ampersands in links
-        # (since links always contain an unknown tag, the "a" tag)
-        txt = original_txt.replace('&amp;', 'XXX_SAVE_AMPERSAND_XXX')
-        txt = txt.replace('&', '&amp;')
-        txt = txt.replace('XXX_SAVE_AMPERSAND_XXX', '&amp;')
-        return txt
+        return original_txt
 
 
 def convert_from_pango(pango_markup):
@@ -316,43 +315,3 @@ def convert_from_pango(pango_markup):
     logging.log(5, 'Converted "%s" pango to "%s" txt2tags' %
                 (repr(original_txt), repr(pango_markup)))
     return pango_markup
-
-
-def get_table_markup(table):
-    '''
-    table is a list of lists
-
-    return the txt2tags markup for that table
-    '''
-    table = map(lambda row: (str(cell) for cell in row), table)
-    table = map(lambda row: '| ' + ' | '.join(row) + ' |', table)
-    table[0] = '|' + table[0]
-    return '\n'.join(table)
-
-
-if __name__ == '__main__':
-    markup = '''\
-normal text, normal_text_with_underscores and ""raw_text_with_underscores""
-
-[Link ""http://www.co.whatcom.wa.us/health/environmental/site_hazard/sitehazard.jsp""]
-'''
-
-    markup =  '[""/image"".png?50]\n'
-    markup += '[""/image"".jpg]\n'
-    markup += '[""file:///image"".png?10]\n'
-    markup += '[""file:///image"".jpg]\n'
-
-    markups = ['http://site/s.php?q&c', 'http://site/s.php?q&amp;c', '&', '&amp;']
-    for markup in markups:
-        print 'MARKUP    ', markup
-        p = convert_to_pango(markup)
-        print 'PANGO     ', p
-        new_markup = convert_from_pango(p)
-        print 'NEW MARKUP', new_markup
-        print
-
-    #html = convert(markup, 'xhtml')
-    #print html
-
-    #latex = convert(markup, 'tex')
-    #print latex
